==> hologram-python-0.6.1.tar.gz: hologram-python-0.6.1/Hologram/Network/Modem/ModemMode/PPP.py <==
# PPP.py - Hologram Python SDK Modem PPP interface
#
# Author: Hologram <support@hologram.io>
#
# Copyright 2016 - Hologram (Konekt, Inc.)
#
#
# LICENSE: Distributed under the terms of the MIT License
#
import psutil
import subprocess
import time
from pppd import PPPConnection
from IPPP import IPPP
from Exceptions.HologramError import PPPError

DEFAULT_PPP_TIMEOUT = 200
DEFAULT_PPP_INTERFACE = 'ppp0'
MAX_PPP_INTERFACE_UP_RETRIES = 10
MAX_REROUTE_PACKET_RETRIES = 15

class PPP(IPPP):

    def __init__(self, device_name='/dev/ttyUSB0', all_attached_device_names=[],
                 baud_rate='9600', chatscript_file=None):

        super(PPP, self).__init__(device_name=device_name, baud_rate=baud_rate,
                                  chatscript_file=chatscript_file)

        self.all_attached_device_names = all_attached_device_names
        self._ppp = PPPConnection(self.device_name, self.baud_rate, 'noipdefault',
                                  'usepeerdns', 'defaultroute', 'persist', 'noauth',
                                  connect=self.connect_script)

    def isConnected(self):
        return self._ppp.connected()

    # EFFECTS: Establishes a PPP connection. If this is successful, it will also
    #          reroute packets to ppp0 interface.
    def connect(self, timeout=DEFAULT_PPP_TIMEOUT):

        self.__enforce_no_existing_ppp_session()

        result = self._ppp.connect(timeout=timeout)

        if result == True and self.__is_ppp_interface_up():
            self.__reroute_packets()
            return True
        else:
            return False

    def disconnect(self):
        self.__shut_down_existing_ppp_session()
        return self._ppp.disconnect()

    # EFFECTS: Blocks to make sure ppp interface is up.
    def __is_ppp_interface_up(self):
        count = 0
        while count <= MAX_PPP_INTERFACE_UP_RETRIES:
            try:
                out_list = subprocess.check_output(['ip', 'address', 'show', 'dev',
                                                    DEFAULT_PPP_INTERFACE],
                                                   stderr=subprocess.STDOUT)

                # Check if ready to break out of loop when ppp0 is found.
                if 'does not exist' in out_list:
                    time.sleep(1)
                    count += 1
                else:
                    break
            except subprocess.CalledProcessError as e:
                pass

        if count <= MAX_PPP_INTERFACE_UP_RETRIES:
            return True
        return False

    # EFFECTS: Makes sure that there are no existing PPP instances on the same
    #          device interface.
    def __enforce_no_existing_ppp_session(self):

        pid_list = self.__check_for_existing_ppp_sessions()

        if len(pid_list) > 0:
            raise PPPError('Existing PPP session(s) are established by pid(s) %s. Please close/kill these processes first'
                             % pid_list)

    def __shut_down_existing_ppp_session(self):
        pid_list = self.__check_for_existing_ppp_sessions()

        # Process this only if it is a valid PID integer.
        for pid in pid_list:
            self.logger.info('Killing pid %s that currently have an active PPP session',
                             pid)
            psutil.Process(pid).terminate()

    def __check_for_existing_ppp_sessions(self):

        existing_ppp_pids = []
        self.logger.info('Checking for existing PPP sessions')

        for proc in psutil.process_iter():

            try:
                pinfo = proc.as_dict(attrs=['pid', 'name'])
            except:
                raise PPPError('Failed to check for existing PPP sessions')

            if 'pppd' in pinfo['name']:
                self.logger.info('Found existing PPP session on pid: %s', pinfo['pid'])
                existing_ppp_pids.append(pinfo['pid'])

        return existing_ppp_pids


    def __reroute_packets(self):
        self.logger.info('Rerouting packets to %s interface', DEFAULT_PPP_INTERFACE)

        count = 0
        # Make sure that we still have ppp interface before adding the routes.
        while count <= MAX_REROUTE_PACKET_RETRIES:
            try:
                out_list = subprocess.check_output(['ip', 'route', 'add',
                                                    '10.176.0.0/16', 'dev',
                                                    DEFAULT_PPP_INTERFACE],
                                                   stderr=subprocess.STDOUT)

                # Check if ready to break out of loop when ppp0 is found.
                if 'Network is down' in out_list:
                    time.sleep(1)
                    count += 1
                else:
                    break
            except Exception as e:
                pass

        if count > MAX_REROUTE_PACKET_RETRIES:
            return

        subprocess.call('ip route add 10.254.0.0/16 dev %s' % DEFAULT_PPP_INTERFACE, shell=True)
        subprocess.call('ip route add default dev %s' % DEFAULT_PPP_INTERFACE, shell=True)

    @property
    def localIPAddress(self):
        return self._ppp.raddr

    @property
    def remoteIPAddress(self):
        return self._ppp.laddr

==> hologram-python-0.7.0.tar.gz: hologram-python-0.7.0/Hologram/Network/Modem/ModemMode/PPP.py <==
# PPP.py - Hologram Python SDK Modem PPP interface
#
# Author: Hologram <support@hologram.io>
#
# Copyright 2016 - Hologram (Konekt, Inc.)
#
#
# LICENSE: Distributed under the terms of the MIT License
#
import psutil
from pppd import PPPConnection
from IPPP import IPPP
from Hologram.Network.Route import Route
from Exceptions.HologramError import PPPError

DEFAULT_PPP_TIMEOUT = 200
DEFAULT_PPP_INTERFACE = 'ppp0'
MAX_PPP_INTERFACE_UP_RETRIES = 10
MAX_REROUTE_PACKET_RETRIES = 15


class PPP(IPPP):

    def __init__(self, device_name='/dev/ttyUSB0', all_attached_device_names=[],
                 baud_rate='9600', chatscript_file=None):

        super(PPP, self).__init__(device_name=device_name, baud_rate=baud_rate,
                                  chatscript_file=chatscript_file)

        self.route = Route()
        self.all_attached_device_names = all_attached_device_names
        self._ppp = PPPConnection(self.device_name, self.baud_rate, 'noipdefault',
                                  'usepeerdns', 'defaultroute', 'persist', 'noauth',
                                  connect=self.connect_script)

    def isConnected(self):
        return self._ppp.connected()

    # EFFECTS: Establishes a PPP connection. If this is successful, it will also
    #          reroute packets to ppp0 interface.
    def connect(self, timeout=DEFAULT_PPP_TIMEOUT):

        self.__enforce_no_existing_ppp_session()

        result = self._ppp.connect(timeout=timeout)

        if result == True and self.route.wait_for_interface(DEFAULT_PPP_INTERFACE,
                                                            MAX_PPP_INTERFACE_UP_RETRIES):
            self.__reroute_packets()
            return True
        else:
            return False

    def disconnect(self):
        self.__shut_down_existing_ppp_session()
        return self._ppp.disconnect()

    # EFFECTS: Makes sure that there are no existing PPP instances on the same
    #          device interface.
    def __enforce_no_existing_ppp_session(self):

        pid_list = self.__check_for_existing_ppp_sessions()

        if len(pid_list) > 0:
            raise PPPError('Existing PPP session(s) are established by pid(s) %s. Please close/kill these processes first'
                             % pid_list)

    def __shut_down_existing_ppp_session(self):
        pid_list = self.__check_for_existing_ppp_sessions()

        # Process this only if it is a valid PID integer.
        for pid in pid_list:
            self.logger.info('Killing pid %s that currently have an active PPP session',
                             pid)
            psutil.Process(pid).terminate()

    def __check_for_existing_ppp_sessions(self):

        existing_ppp_pids = []
        self.logger.info('Checking for existing PPP sessions')

        for proc in psutil.process_iter():

            try:
                pinfo = proc.as_dict(attrs=['pid', 'name'])
            except:
                raise PPPError('Failed to check for existing PPP sessions')

            if 'pppd' in pinfo['name']:
                self.logger.info('Found existing PPP session on pid: %s', pinfo['pid'])
                existing_ppp_pids.append(pinfo['pid'])

        return existing_ppp_pids

    def __reroute_packets(self):
        self.logger.info('Rerouting packets to %s interface', DEFAULT_PPP_INTERFACE)
        # Make sure that we still have ppp interface before adding the routes.
        ppp_available = self.route.wait_for_interface(DEFAULT_PPP_INTERFACE, MAX_REROUTE_PACKET_RETRIES)
        if not ppp_available:
            return
        self.route.add('10.176.0.0/16', self.localIPAddress)
        self.route.add('10.254.0.0/16', self.localIPAddress)
        self.route.add_default(self.localIPAddress)

    @property
    def localIPAddress(self):
        return self._ppp.raddr

    @property
    def remoteIPAddress(self):
        return self._ppp.laddr
